/*
 * module-info
 *
 * Author: Lasse Collin <lasse.collin@tukaani.org>
 *
 * This file has been put into the public domain.
 * You can do whatever you want with this file.
 */

module org.tukaani.xz {
    exports org.tukaani.xz;
}
